/*************************************************************
Project :  Test des fonctionnalits de la carte ATMEGA8535 SSI
Create :  8/03/2003 pour AT90S8535
Version : 1
Rvision : 6     Date    : 15/03/2008
Author  : Philippe Mariano                
Company : LYCEE Pierre Emile MARTIN       
Comments: Test des BP, des DELs, des entres TOR, du Buzzer,
de l'entre analogique, du capteur de temprature DS1621,
de l'EEPROM externe, de l'HTR PCF8583, de la liaison RS232, de la sortie 
PWMA, d'un clavier 102 touches

Chip type           : ATMEGA8535
Clock frequency     : 8,000000 MHz
Memory model        : Small
Internal SRAM size  : 512
External SRAM size  : 0
Data Stack size     : 152 !!!!!!!!!!!!!!!!!
************************************************************/

#include <mega8535.h>

// Librairies non gnres par Wizard ( rajouter manuellement)
//-----------------------------------------------------------------      
#include <delay.h>    // contient la fonction delay_ms
#include <stdio.h>    // contient la fonction sprintf 
#include <stdlib.h>   // contient la fonction itoa
#include <math.h>

// Ces fichiers d'en-tte et les librairies correspondantes doivent tre placs
// dans les rpertoires c:/cvavr/inc et c:/cvavr/lib
// -------------------------------------------------------------------------------
#include <ssi.h> 	// Sous-programme Affiche_LCD, Lire_BP  

#include <LitClav102.h> // Sous-programme de dsrialisation et de dcodage de la
			// trame issue du clavier de PC (102T) 
#include <scancode2.h> 	// Table de transcodage du clavier de PC (102T)
 			// Cette table doit tre adapte au clavier utilis 
//**********************************************************************************     

// Boolens
// --------------------------------
#define VRAI 1
#define FAUX 0

// Accs aux caractres en CGRAM 
// --------------------------------
#define Caractere_FlecheH	0 
#define Caractere_FlecheB	1

// @ I2C HTR
// --------------------------------
#define	adresse_HTR	0

// Adresse CGRAM de l'afficheur LCD
// --------------------------------
#define Add_FlecheH_CGRAM 0x40 // @1 Caractre flche haut
#define Add_FlecheB_CGRAM 0x48 // @2 Caractre flche bas


// ----------------------------------------  page 1 ------------------------------------------------


// I2C Bus functions
#asm
   .equ __i2c_port=0x12
   .equ __sda_bit=3
   .equ __scl_bit=7
#endasm
#include <i2c.h>

// DS1621 Thermometer/Thermostat functions
#include <ds1621.h>

// PCF8583 Real Time Clock functions
#include <pcf8583.h>

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>

#define ADC_VREF_TYPE 0x00
// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input|ADC_VREF_TYPE;
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}


// Declare your global variables here   
// --------------------------------------------------------------------------------------
/* Communication avec le clavier PS2
Le programme principal se synchronise avec la routine d'interruption situe dans 
la bibliothque LitClav.lib avec les variables ci-dessous
----------------------------------------------------------------------------------------*/ 
unsigned char 	g_key_ASCII=''; 	// Contient le code ASCII de la touche presse  
					// ( initialiser avec uncaractre n'existant 
					// pas sur le clavier utilis)
bit g_flag_char = 0;			// g_flag_char = 1 lorsqu'une touche est appuye
					// g_flag_char = 0 lorsqu'une touche a t relche
bit g_Sens_transfert = 0;		// Indique le sens de la communication  
					// g_Sens_transfert = 0 => C <--- clavier
			  		// g_Sens_transfert = 1 => C ---> clavier
unsigned int g_Trame_TX;                // ?
/* --------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------
type 		nom 				       		dsignation       
------------------------------------------------------------------------------------------*/
char display_buffer_ligne0[17]; 			// tampon ligne 0 de l'afficheur
char display_buffer_ligne1[17]; 	       		// tampon ligne 1 de l'afficheur
unsigned char heures, minutes, secondes, secondes_1, dizieme; 	//stockage de l'heure
unsigned char Heure_ou_Date, H_J, mn_mois;
unsigned int sec_annee;

enum Etat_Test {LedBP,LedTOR,Analog,Temperature,  	// Etats du graphe tests
 	Reglage_HTR,Horloge,Buzzer,EEPROM,RS232Tx,MOT_PWM,REG_PWM, 
 	CLAV_PC,REG_TST_CLAV102,TST_CLAV_102T } etat=LedBP, etat_1=LedTOR; 	 
// ----------------------------------------  page 2 ------------------------------------------------
 
// Constantes en flash
const char menu[]	 = "  [ENTR]>Oui    ";      
const char menu_BP[]     = "Test? BP->DEL   "; 
const char menu_TOR[] 	 = "Test? TOR->DEL  ";
const char menu_Analog[] = "Test? E_Analog  "; 
const char menu_Buzzer[] = "Test? Buzzer    "; 
const char menu_EEPROM[] = "Test? EEPROM    ";  
const char menu_RS232[]  = "Test? RS232Tx   "; 
const char menu_PWM[]    = "Test? PWMA      ";
const char menu_PC[]     = "Tst? Clav102 PC "; 
const char menu_ECHAP[]  = "[ECHAP]>Quitter ";

// Caractres personnaliss placs dans la mmoire CGRAM du LCD  
const unsigned char flecheHaut[8] = {0x04, 0x0E, 0x15, 0x04, 0x04, 0x04, 0x04, 0x00};    // @1 CGRAM
const unsigned char flecheBas[8] = {0x04, 0x04, 0x04, 0x04, 0x15, 0x0E, 0x04, 0x00};     // @2 CGRAM       


// Prototypes des fonctions
int Led_BP_TOR(char entree);  // test des Leds et des BPs 
void Affiche_fleches(void);

//-----------------------
// Programme principal
//-----------------------
void main(void)
{ 

// Declare your local variables here
// -----------------------------------------------------------------------------------------
//type 		nom 					dsignation  
// -----------------------------------------------------------------------------------------
char display_buffer[32]; 		// Tampon utilis lors de la lecture de la EEPROM
			 		// en page de 32 octets
unsigned char i, j;                     // Variable d'ordre gnrale
unsigned char BP;			// Reoit le code de la touche actionne
unsigned char jour,mois;
unsigned int  temp_n,annee;  
unsigned int  mesure,analogU, analogD, analogC; // Calcul de la ddp dlivre par le pot
unsigned char *Transm; 
unsigned char rcycl=0, rcycl_1=0;
// Coefficient pour le rglage du rapport cyclique du signal PWM
unsigned char alpha[11] = {0,25,51,76,102,127,153,178,204,229,255};   

// Input/Output Ports initialization
// Port A
PORTA=0x3F; // Port A en entre avec rsistance de rappel
DDRA=0x00;

// Port B
PORTB=0x00; // Port B en sortie
DDRB=0x1F;

// Port C
PORTC=0x00; // Port C en sortie
DDRC=0xFF;  // Partag entre le LCD et le Buzzer(PORTC.3)

// Port D initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=Out Func6=In Func7=In
// State0=T State1=T State2=T State3=T State4=T State5=0 State6=T State7=T
PORTD=0x00;
DDRD=0x20;

// ----------------------------------------  page 3 ------------------------------------------------


// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 8000,000 kHz
// Mode: Ph. correct PWM top=00FFh
// OC1A output: Non-Inv.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x81;
TCCR1B=0x01;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// ADC initialization
// ADC Clock frequency: 62,500 kHz
// ADC Voltage Reference: AREF pin
// ADC High Speed Mode: Off
// ADC Auto Trigger Source: None
ADMUX=ADC_VREF_TYPE;
ADCSRA=0x87;
SFIOR&=0xEF;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: Off
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud rate: 19200
UCSRA=0x00;
UCSRB=0x08;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x19;

// I2C Bus initialization
i2c_init();

// DS1621 Thermometer/Thermostat initialization
// tlow: 24C
// thigh: 55C
// Tout polarity: 0
ds1621_init(0,24,55,0);

// PCF8583 Real Time Clock initialization
rtc_init(0,0);

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x02;
MCUCSR=0x00;
GIFR=0x40;

// ----------------------------------------  page4 ------------------------------------------------


// LCD module initialization
lcd_init(16); 

// Chargement initial de la mmoire CGRAM de l'afficheur avec les 
// caractres personaliss
for (i=0;i<8;i++)
{
lcd_write_byte(Add_FlecheH_CGRAM + i, flecheHaut[i]); // @1 Caractre flche haut
lcd_write_byte(Add_FlecheB_CGRAM + i, flecheBas[i]); // @2 Caractre flche bas
}

sprintf(display_buffer_ligne0,"Test de la carte");
sprintf(display_buffer_ligne1,"******S.SI******");
Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
delay_ms(3000);

// Global enable interrupts
#asm("sei")

while (1)
      {		   // Dbut While(1)  
       
      BP = Lire_BP();
      switch (etat)
       {               // Dbut switch (etat)
        case LedBP :   if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_BP);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
                        switch(BP)
        		    {
        		     case INC : etat = LedTOR; break;
        		     case DEC : etat = CLAV_PC; break;
        		     case ENTR: Led_BP_TOR(0); //test des BP
       		      			sprintf(display_buffer_ligne0,menu_BP);
                  			sprintf(display_buffer_ligne1,menu);
                     			Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                     			Affiche_fleches();
                             break;
        		     default  : etat = LedBP;
        		    }
       			etat_1 = LedBP;
       		        
        break;
        
        case LedTOR : if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_TOR);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches();
                        } 
                        switch(BP)
        		    {
        		     case INC : etat = Buzzer; break;
        		     case DEC : etat = LedBP; break;
        		     case ENTR: Led_BP_TOR(1); // test des entres TOR
       		      			sprintf(display_buffer_ligne0,menu_TOR);
                  			sprintf(display_buffer_ligne1,menu);
                     			Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                     			Affiche_fleches(); 
                             break;
        		     default  : etat = LedTOR;
        		    }
       			etat_1 = LedTOR;	
        break;
        
        case Buzzer:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_Buzzer);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
        		switch(BP)
        		    {
        		     case INC : etat = Analog; break;
        		     case DEC : etat = LedTOR; break;
        		     default  : etat = Buzzer;
        		    }
       			PORTC.3 = !PINA.0;
       			etat_1 = Buzzer;       		
        break;  
        
        case Analog :  	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_Analog);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
                        switch(BP)
                         {
        		     case INC : etat = Temperature; break;
        		     case DEC : etat = Buzzer; break;
        		     case ENTR: sprintf(display_buffer_ligne1,menu_ECHAP);
       		       			          
       		      			while (BP != ECHAP) 
       		       			{ 
       		       			 BP = Lire_BP();
       		       			 delay_ms(500);
       		       			 mesure = (read_adc(6)/4); // Mesure et calcul de la ddp
       		       			 analogU = ((mesure*5)/255);
       		       			 analogD = ((10*((mesure - (analogU*255)/5))*5)/255);
       		       			 analogC = ((100*(mesure - (analogU*255)/5 -(analogD*255)/50)*5)/255);
       		       			 sprintf(display_buffer_ligne0,"  DDP = %-u.%-u%-uV",analogU, analogD, analogC);
       		       			 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		       			}
       		       			 sprintf(display_buffer_ligne0,menu_Analog);
                    			 sprintf(display_buffer_ligne1,menu);
                        		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);  
       		             break;
       			     default  : etat = Analog;
       			   }
       			etat_1 = Analog;       		
        break;
        
        case Temperature : temp_n = ds1621_temperature_10(0); // Affichage de la temprature toutes les secondes
        	           rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);
        	       	   if ((etat_1 != etat) || (secondes_1 != secondes))
        		    {  
        		      sprintf(display_buffer_ligne0,"Capteur DS1621  ");
        		      sprintf(display_buffer_ligne1,"Temp. = %-i.%-u%-cC",temp_n/10,abs(temp_n%10),0xdf);
        		      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        	              secondes_1 = secondes;   
        	              Affiche_fleches();
        	            }
        	           switch(BP)
        		    {
        		     case INC : etat = EEPROM; break;
        		     case DEC : etat = Analog; break;
        		     default  : etat = Temperature;
        		    }
        		   etat_1 = Temperature;
        break;
        
        case EEPROM:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_EEPROM);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = Horloge; break;
        		     case DEC : etat = Temperature; break;
        		     case ENTR: j=0;
       		      		        for (i=0;i<8;i++)
					 { 
 					   lcd_clear();
 					   eeprom_page_read(0xae,j,display_buffer); // EEPROM  l'@AE
 					   lcd_puts(display_buffer);
 					   delay_ms(4000);
 					   j=j+32;
					 }
        		     default  : etat = EEPROM;
        		    }
       			etat_1 = EEPROM;       		
        break;            
        
        case Horloge :  rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);    
        		if ((etat_1 != etat) || (secondes_1 != secondes))
        		{
        		 rtc_get_date(adresse_HTR,&jour,&mois,&annee);
        		 sprintf(display_buffer_ligne0," %-u / %-u / %-i",jour,mois,annee);
        		 sprintf(display_buffer_ligne1,"%-uh / %-umn / %-us ",heures, minutes, secondes);
        		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        	         secondes_1 = secondes;
        	         Affiche_fleches();
        	        }
        		switch(BP)
        		{
        		 case INC : etat = RS232Tx; break;
        		 case DEC : etat = EEPROM; break;
        		 case ENTR : etat = Reglage_HTR; break; 
        		 default : etat = Horloge;
        		}
        		etat_1 = Horloge;
        break;     
        
        case Reglage_HTR : if (etat_1 != etat)
                           {
        	            sprintf(display_buffer_ligne0,"  Reglage HTR");
        	            sprintf(display_buffer_ligne1,"SET>H ETR>Date ");
        	            Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        		   }   
        		               
        	          switch(BP)
        		   {
        		    case SET   : Heure_ou_Date = 0;
        		  	        H_J = 12;
        		  	        mn_mois = 30;
        		  	        sec_annee = 0;
        		  	        regle_HTR(adresse_HTR,Heure_ou_Date,H_J,mn_mois,sec_annee);
        		  	        etat = Horloge;
        		    break;
        		 
        		    case ENTR : Heure_ou_Date = 1;
        		  	         H_J = 15;
        		  	         mn_mois = 6;
        		  	         sec_annee = 2008;
        		  	         regle_HTR(adresse_HTR,Heure_ou_Date,H_J,mn_mois,sec_annee);
        		  	         etat = Horloge;
        		    break;
        		 
        		    default:     etat = Reglage_HTR;
        		   } 
        	          etat_1 = Reglage_HTR;
        break; 
        
        case RS232Tx:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_RS232);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = MOT_PWM; break;
        		     case DEC : etat = Horloge; break;
        		     case ENTR: rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);
        		     sprintf(Transm,"%u/%u/%u",heures, minutes, secondes);
        		     puts(Transm); 
        		     break;       
        		     default  : etat = RS232Tx;
        		    }
       			etat_1 = RS232Tx;       		
        break; 
        
        case MOT_PWM :	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_PWM);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = CLAV_PC; break;
        		     case DEC : etat = RS232Tx; break;
        		     case ENTR: etat = REG_PWM; break; 
        		     default  : etat = MOT_PWM;
        		    }
       			etat_1 = MOT_PWM;       
        break;
        
        case REG_PWM :  if ((etat_1 != etat)|| (rcycl_1 != rcycl))
       			  {
       		   	    sprintf(display_buffer_ligne0,"Rap. cycl = %-u%%",rcycl);
       		            sprintf(display_buffer_ligne1,"INC>+ DEC>- ESc>");
       		            Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		            rcycl_1 = rcycl; 
			  }
		        switch(BP)
			      {
				case INC: if (rcycl < 90) rcycl = rcycl+10; else rcycl = 100; break;
				case DEC: if (rcycl > 10) rcycl = rcycl-10; else rcycl = 0; break;
				case ECHAP : rcycl = 0; etat = MOT_PWM; break;
				default: etat = REG_PWM;
                              }
			OCR1AL=alpha[rcycl/10];
         		etat_1 = REG_PWM ;      		      
        break;
        
        case CLAV_PC :	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menu_PC);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = LedBP; break;
        		     case DEC : etat = MOT_PWM; break;
        		     case ENTR: etat = REG_TST_CLAV102; break; 
        		     default  : etat = CLAV_PC;
        		    }
       			etat_1 = CLAV_PC;       
        break; 
        
        case REG_TST_CLAV102 :	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,"Caval PAO 1-2 OK");
                         sprintf(display_buffer_ligne1,menu_ECHAP);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                        }  
                        switch(BP)
        		    {
        		     case OK : etat = TST_CLAV_102T; break;
        		     case ECHAP: etat = CLAV_PC; break; 
        		     default  : etat = REG_TST_CLAV102;
        		    }
       			etat_1 = REG_TST_CLAV102;       
        break; 
        
        case TST_CLAV_102T : if (etat_1 != etat)
                       {
                        sprintf(display_buffer_ligne0,"                ");
                    	sprintf(display_buffer_ligne1,menu_ECHAP);
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(0,0);_lcd_write_data(displayOn3); i =0; 
                       }
                      
                      if (g_flag_char == VRAI)
                       { 
                         if (i<16)
                               {
                         	lcd_putchar(g_key_ASCII);
                         	i++;
                               } 
                         else 
                         	{ 
                         	  sprintf(display_buffer_ligne0,"                ");
                       		  sprintf(display_buffer_ligne1,menu_ECHAP);
                    		  Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    		  lcd_gotoxy(0,0);_lcd_write_data(displayOn3); i = 0;      
                         	}
                         g_flag_char = FAUX;
                       }  
                       
                      switch(BP)
        		    {
        		     case ECHAP: etat = CLAV_PC; break; 
        		     default  : etat = TST_CLAV_102T;
        		    } 
       		    etat_1 = TST_CLAV_102T;
       break;           
        
       }; // Fin switch(etat)
      }  // Fin while(1)    
} // Fin main()

//===================================================================
// Sous-programmes
//-------------------------------------------------------------------
void Affiche_fleches(void)
{
lcd_gotoxy(15,0);	// Affichage flecheHaut
		         lcd_putchar(Caractere_FlecheH);	
			 lcd_gotoxy(15,1);	// Affichage flecheBas
			 lcd_putchar(Caractere_FlecheB);
}

// -------------------------------------------------------------------

int Led_BP_TOR(char entree)
{
  unsigned char BP, Affich=1, sortir=0;
  
  if (entree==0)
       {
        sprintf(display_buffer_ligne1,"Sortir = 2xECHAP");
    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       }
        else 
       {
        sprintf(display_buffer_ligne1,"Sortir = 2xPA2  ");
       	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       }
  
  while(sortir<2)
  {
   BP=Lire_BP();
   switch (BP)
      {
       case INC: PORTB = 0x1F;
       		 sprintf(display_buffer_ligne0,"   PA5 -> ALL   ");
       		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                 Affich = 1;
                 if (sortir=1) sortir=0;
       break; 
       
       case DEC: PORTB.4 = 1;
       		 sprintf(display_buffer_ligne0,"   PA4 -> PB4   ");
       	       	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1;
                 if (sortir=1) sortir=0;
       	break;
       	 
     	case OK: PORTB.3 = 1;
     		 sprintf(display_buffer_ligne0,"   PA3 -> PB3   ");
       	       	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1; 
                 if (sortir=1) sortir=0;
       	break;
       	  
      	case SET:PORTB.1 = 1;
      		 sprintf(display_buffer_ligne0,"   PA1 -> PB1   ");
        	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1;
                 if (sortir=1) sortir=0;
       	break;
       	   
      	case ECHAP: PORTB.2 = 1;
      	            sprintf(display_buffer_ligne0,"   PA2 -> PB2   ");
       	       	    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		    Affich = 1;
       		    sortir = sortir + 1;
       	break;
       	
       	case ENTR: PORTB.0 = 1;
       		    sprintf(display_buffer_ligne0,"   PA0 -> PB0   ");
       	       	    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                    Affich = 1;
                    if (sortir=1) sortir=0;
       	break;
       	     
      	case 0xff: PORTB = 0;
      	           if ((Affich == 1)&&(entree==0))
       	  	    {
       	  	      sprintf(display_buffer_ligne0," Test BPx->DELx");
       	       	      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       	       	    }
       	       	    else if  ((Affich == 1)&&(entree==1))
       	       	    {
       	  	      sprintf(display_buffer_ligne0,"Test TORx->DELx");
       	       	      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       	       	    }
       	       	    Affich = 0; 
       	       	    
       	break;
       }
  }
PORTB = 0;   
return 0;
}

